import numpy as np
from i1d import intersect_1d

size = 10_000
iters = 50

a1k = np.random.choice(10_000, size=1_000, replace=False)
a100k = np.random.choice(1_000_000, size=100_000, replace=False)
a5m = np.random.choice(20_000_000, size=5_000_000, replace=False)

b1k = np.random.choice(10_000, size=1_000, replace=False)
b100k = np.random.choice(1_000_000, size=100_000, replace=False)
b5m = np.random.choice(20_000_000, size=5_000_000, replace=False)


def test_same_as_numpy():

    for _ in range(iters):
        a = np.random.choice(size * 10, size=size, replace=False)
        b = np.random.choice(size * 10, size=size, replace=False)

        i1, j1 = intersect_1d(a, b)
        i2, j2 = np.intersect1d(a, b, return_indices=True, assume_unique=True)[1:]

        assert np.all(np.equal(np.sort(i1), np.sort(i2)))
        assert np.all(np.equal(np.sort(j1), np.sort(j2)))


def test_1k_i1d(benchmark):
    benchmark(intersect_1d, a1k, b1k)


def test_1k_i1d_par(benchmark):
    benchmark(intersect_1d, a1k, b1k, parallel=True)


def test_1k_numpy(benchmark):
    benchmark(np.intersect1d, a1k, b1k, return_indices=True, assume_unique=True)


def test_100k_i1d(benchmark):
    benchmark(intersect_1d, a100k, b100k)


def test_100k_i1d_par(benchmark):
    benchmark(intersect_1d, a100k, b100k, parallel=True)


def test_100k_numpy(benchmark):
    benchmark(np.intersect1d, a100k, b100k, return_indices=True, assume_unique=True)


def test_5m_i1d(benchmark):
    benchmark(intersect_1d, a5m, b5m)


def test_5m_i1d_par(benchmark):
    benchmark(intersect_1d, a5m, b5m, parallel=True)


def test_5m_numpy(benchmark):
    benchmark(np.intersect1d, a5m, b5m, return_indices=True, assume_unique=True)



from typing import Tuple
from .i1d import *
import numpy as np


__doc__ = i1d.__doc__
__all__ = ["intersect_1d"]


def _intersect_1d(a, b, parallel):
    assert isinstance(a, np.ndarray) and isinstance(
        b, np.ndarray
    ), "a and b must be numpy arrays"
    assert a.dtype == b.dtype, "a and b must have the same dtype"
    assert a.ndim == 1 and b.ndim == 1, "a and b must be 1D arrays"
    if parallel:
        if a.dtype == np.int64:
            return par_i1d_i64(a, b)
        elif a.dtype == np.int32:
            return par_i1d_i32(a, b)
        elif a.dtype == np.int16:
            return par_i1d_i16(a, b)
        elif a.dtype == np.int8:
            return par_i1d_i8(a, b)
        elif a.dtype == np.uint64:
            return par_i1d_u64(a, b)
        elif a.dtype == np.uint32:
            return par_i1d_u32(a, b)
        elif a.dtype == np.uint16:
            return par_i1d_u16(a, b)
        elif a.dtype == np.uint8:
            return par_i1d_u8(a, b)
        raise ValueError("Unsupported dtype: {}".format(a.dtype))
    else:
        if a.dtype == np.int64:
            return i1d_i64(a, b)
        elif a.dtype == np.int32:
            return i1d_i32(a, b)
        elif a.dtype == np.int16:
            return i1d_i16(a, b)
        elif a.dtype == np.int8:
            return i1d_i8(a, b)
        elif a.dtype == np.uint64:
            return i1d_u64(a, b)
        elif a.dtype == np.uint32:
            return i1d_u32(a, b)
        elif a.dtype == np.uint16:
            return i1d_u16(a, b)
        elif a.dtype == np.uint8:
            return i1d_u8(a, b)
        raise ValueError("Unsupported dtype: {}".format(a.dtype))


def intersect_1d(
    a: np.ndarray, b: np.ndarray, parallel: bool = False
) -> Tuple[np.ndarray, np.ndarray]:
    """
    Returns two arrays containing the indices of the intersection of a and b.

    Parameters
    ----------
    a : numpy.ndarray
        1D array of integers.
    b : numpy.ndarray
        1D array of integers.
    parallel : bool, optional
        If True, the intersection is computed in parallel using Rayon.

    Returns
    -------
    a_idx : numpy.ndarray
        1D array of indices of the intersection of a and b.
    b_idx : numpy.ndarray
        1D array of indices of the intersection of b and a.

    Examples
    --------
    >>> a = np.array([1, 2, 3, 4, 5])
    >>> b = np.array([2, 3, 4, 5, 6])
    >>> a_idx, b_idx = intersect_1d(a, b)
    >>> a_idx
    array([1, 2, 3, 4])
    >>> b_idx
    array([0, 1, 2, 3])
    """
    a_ix, b_ix = _intersect_1d(a, b, parallel)
    return a_ix, b_ix
